# -*- coding: utf-8 -*-
import unittest
from khayyam import JalaliDatetime, teh_tz
from datetime import datetime, timedelta
from khayyam.timezones import TehranTimezone
from khayyam.jalali_date import JalaliDate
__author__ = 'vahid'


class TestJalaliDateTime(unittest.TestCase):
    
    def setUp(self):
        self.leap_year = 1375
        self.naive_jdt = JalaliDatetime(self.leap_year, 12, 30, 10, 2, 1, 3)
        self.aware_jdt = JalaliDatetime(self.leap_year, 12, 30, 10, 2, 1, 3, TehranTimezone())

    def test_instantiate(self):

        jtime = JalaliDatetime(1376, 10, 9, 8, 7, 6, 5)
        self.assertFalse(jtime is None)

        self.assertEqual(JalaliDatetime(jtime.todatetime()), jtime)
        self.assertEqual(JalaliDatetime(jtime), jtime)
        self.assertEqual(JalaliDatetime(jtime.date()).date(), jtime.date())

        self.assertEqual(JalaliDatetime(julian_day=2450674), JalaliDatetime(1376, 5, 23))


    def test_to_from_datetime(self):
        # Naive
        jdate1 = JalaliDatetime(self.naive_jdt.todatetime())
        self.assertEqual(self.naive_jdt, jdate1)
        
        # Aware
        jdate2 = JalaliDatetime(self.aware_jdt.todatetime())
        self.assertEqual(self.aware_jdt, jdate2)
        
    def test_today(self):
        dt = datetime.now().date()
        jdt = JalaliDatetime.today().date()
        self.assertEqual(jdt, JalaliDate(dt))
        
    def test_now(self):
        self.assertIsNotNone(JalaliDatetime.now())
        self.assertIsNone(JalaliDatetime.now().tzinfo)
        self.assertIsNotNone(JalaliDatetime.now(TehranTimezone()).tzinfo)
        
    def test_utcnow(self):
        jutcnow = JalaliDatetime.utcnow()
        utcnow = jutcnow.todatetime()
        self.assertEqual(jutcnow.time(), utcnow.time())
    
    def test_strftime_strptime(self):
        d1 = JalaliDatetime(self.leap_year, 12, 23, 12, 3, 45, 34567)
        self.assertEqual(d1.strftime('%Q'), 'Panjshanbeh 23 Esfand 1375 12:03:45 PM')
        self.assertEqual(d1.isoformat(), '%s-12-23T12:03:45.034567' % self.leap_year)
        tz_datetime = d1.astimezone(teh_tz)
        self.assertEqual(tz_datetime.strftime('%Z'), 'Iran/Tehran')


    def test_iso_format(self):
        jdate = JalaliDatetime(self.leap_year, 12, 23)
        self.assertEqual(jdate.isoformat(), '%s-12-23T00:00:00.000000' % self.leap_year)
        jdate = JalaliDatetime(self.leap_year, 12, 23, tzinfo=teh_tz)
        self.assertEqual(jdate.isoformat(), '%s-12-23T00:00:00.000000+03:30' % self.leap_year)

    def test_algorithm(self):
        min = datetime(1900, 1, 1, 1, 1, 1)
        max_days = 5000 # 3000 years !
        days = 0
        while True:
            dt = min + timedelta(days=days)
            jd = JalaliDatetime(dt)
            # print('Processing day: %s' % jd.year)
            dt2 = jd.todatetime()
            self.assertEqual(dt, dt2)
            days += 1
            if days > max_days:
                break

    def test_add(self):
        jdate = JalaliDatetime(self.leap_year, 12, 23)
        jdate2 = jdate + timedelta(10)
        
        self.assertEqual(jdate2, JalaliDatetime(self.leap_year + 1, 1, 3))
        
    def test_sub(self):
        jdate = JalaliDatetime(self.leap_year, 12, 23)
        jdate2 = jdate - timedelta(10)
        
        self.assertEqual(jdate2, JalaliDatetime(self.leap_year, 12, 13))

        jtimedelta = jdate - JalaliDatetime(self.leap_year - 1, 12, 1)

        self.assertEqual(jtimedelta, timedelta(387))

        jdate = JalaliDatetime(self.leap_year, 12, 23, 4, 2, 10, 7)
        self.assertEqual(jdate - jdate.date(), timedelta(hours=4, minutes=2, seconds=10, microseconds=7))

    def test_lt_gt_le_ge_ne_eg(self):
        jdate = JalaliDatetime(self.leap_year, 12, 23)
        jdate2 = JalaliDatetime(self.leap_year, 12, 24)
        jdate3 = JalaliDatetime(self.leap_year, 12, 24)
        
        self.assertTrue(jdate <= jdate2)
        self.assertTrue(jdate != jdate2)
        self.assertFalse(jdate > jdate2)
        self.assertTrue(jdate2 == jdate3)

    def test_replace(self):
        d1 = JalaliDatetime(1391, 12, 30)
        self.assertEqual(d1.replace(year=1395), JalaliDatetime(1395, 12, 30))
        self.assertEqual(d1.replace(month=1),   JalaliDatetime(1391, 1, 30))
        self.assertEqual(d1.replace(day=1),     JalaliDatetime(1391, 12, 1))
        self.assertRaises(ValueError, d1.replace, year=1392)

    def test_repr(self):
        d1 = JalaliDatetime(self.leap_year, 12, 23, 12, 3, 45, 34567)
        self.assertEqual(repr(d1), 'khayyam.JalaliDatetime(1375, 12, 23, 12, 3, 45, 34567, Panjshanbeh)')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from .directive import Directive
from khayyam.compat import get_unicode
from khayyam.formatting.constants import PERSIAN_DIGIT_MAPPING

__author__ = 'vahid'


eng_to_persian_dict = {e: p[0] for p, e in PERSIAN_DIGIT_MAPPING}
persian_to_eng_dict = {p[0]: e for p, e in PERSIAN_DIGIT_MAPPING}

eng_to_persian = lambda s: ''.join([eng_to_persian_dict[c] if c in eng_to_persian_dict else c for c in s])
persian_to_eng = lambda s: ''.join([persian_to_eng_dict[c] if c in persian_to_eng_dict else c for c in s])


class PersianNumberDirective(Directive):
    def __init__(self, key, name, regex, zero_padding=False, zero_padding_length=2):
        self.zero_padding = zero_padding
        self.zero_padding_length = zero_padding_length
        super(PersianNumberDirective, self).__init__(key, name, regex, get_unicode)

    def format(self, i):
        if self.zero_padding:
            fmt = '%%.%dd' % self.zero_padding_length
            return eng_to_persian(fmt % i)
        elif isinstance(i, int):
            return eng_to_persian(str(i))
        else:
            return eng_to_persian(i)

    def post_parser(self, ctx, formatter):
        exp = ctx[self.name]
        if exp.strip() != '':
            ctx[self.name] = int(persian_to_eng(exp))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import datetime
import time
from khayyam.helpers import force_encoded_string_output
from khayyam.algorithms_pure import days_in_month, \
    is_leap_year, \
    get_julian_day_from_gregorian, \
    jalali_date_from_julian_day, \
    julian_day_from_jalali_date, \
    gregorian_date_from_julian_day
from khayyam import MAXYEAR, MINYEAR, SATURDAY
from khayyam.formatting import \
    JalaliDateFormatter, \
    PERSIAN_MONTH_ABBRS, \
    PERSIAN_MONTH_NAMES, \
    PERSIAN_WEEKDAY_ABBRS, \
    PERSIAN_WEEKDAY_NAMES, \
    PERSIAN_MONTH_ABBRS_ASCII, \
    PERSIAN_MONTH_NAMES_ASCII, \
    PERSIAN_WEEKDAY_ABBRS_ASCII, \
    PERSIAN_WEEKDAY_NAMES_ASCII

__author__ = 'vahid'


class JalaliDate(object):
    """
    Represent a day in :doc:`/persiancalendar`.

    The first parameter can be an integer, :py:class:`datetime.date` or :py:class:`khayyam.JalaliDate`.

    You may create this object by passing `julian_day` parameter.


    .. doctest::

        >>> from khayyam import JalaliDate
        >>> JalaliDate(julian_day=2445218)
        khayyam.JalaliDate(1361, 6, 15, Doshanbeh)

        >>> from datetime import date
        >>> JalaliDate(date(1982, 9, 6))
        khayyam.JalaliDate(1361, 6, 15, Doshanbeh)

        >>> JalaliDate(1361, 6, 15)
        khayyam.JalaliDate(1361, 6, 15, Doshanbeh)


    :param year: jalali year
    :param month: month 1-12
    :param day: day of month
    :param julian_day: julian day

    :type year: :py:class:`int` | :py:class:`datetime.date` | :py:class:`khayyam.JalaliDate`
    :type month: int
    :type day: int
    :type julian_day: int

    :return: A :py:class:`khayyam.JalaliDate` instance.
    :rtype: :py:class:`khayyam.JalaliDate`
    """

    #: Represent the minimum year which supported by this class.
    min = (MINYEAR, 1, 1)   # To be converted to JalaliDate at the bottom of this module

    #: Represent the maximum year which supported by this class.
    max = (MAXYEAR, 12, 29)

    resolution = datetime.timedelta(days=1)

    def __init__(self, year=1, month=1, day=1, julian_day=None):
        if isinstance(year, JalaliDate):
            jd = year
            year = jd.year
            month = jd.month
            day = jd.day
        elif isinstance(year, datetime.date):
            julian_day = get_julian_day_from_gregorian(year.year, year.month, year.day)

        if julian_day is not None:
            year, month, day = jalali_date_from_julian_day(julian_day)

        self.year, self.month, self.day = self._validate(year, month, day)

    @property
    def isleap(self):
        """
        `True` if the current instance is in a leap year.

        :type: bool
        """
        return is_leap_year(self.year)

    @property
    def daysinmonth(self):
        """
        Total days in the current month.

        :type: int
        """
        return days_in_month(self.year, self.month)

    @staticmethod
    def formatterfactory(fmt):
        """
        By default it will be return a :py:class:`khayyam.formatting.JalaliDateFormatter`
        instance based on given format string.

        :param fmt: see: :doc:`/directives`
        :type fmt: str
        :return: Formatter object, based on the given format string.
        :rtype: khayyam.formatting.BaseFormatter
        """
        return JalaliDateFormatter(fmt)

    @classmethod
    def today(cls):
        """
        :return: The current local date.
        :rtype: :py:class:`khayyam.JalaiDate`
        """
        return cls(datetime.date.today())

    @classmethod
    def fromtimestamp(cls, timestamp):
        """
        Such as is returned by :func:`time.time()`. This may raise :class:`ValueError`,
        if the timestamp is out of the range of values supported by the platform C localtime()
        function. It’s common for this to be restricted to years from 1970 through 2038.
        Note that on non-POSIX systems that include leap seconds in their notion of a
        timestamp, leap seconds are ignored by fromtimestamp().

        :return: Local date corresponding to the POSIX timestamp
        :rtype: :py:class:`khayyam.JalaiDate`

        """
        return cls(datetime.date.fromtimestamp(timestamp))

    @classmethod
    def fromordinal(cls, ordinal):
        """
        Where Farvardin 1 of year 1 has ordinal 1.

        ValueError is raised unless 1 <= ordinal <= `khayyam.jalaliDate(khayyam.MAXYEAR).toordinal()`.

        :return: The date corresponding to the proleptic Shamsi ordinal.
        :rtype: :py:class:`khayyam.JalaiDate`
        """
        return cls.min + datetime.timedelta(days=ordinal-1)

    @classmethod
    def strptime(cls, date_string, format):
        """
        This is opposite of the :py:meth:`khayyam.JalaliDate.strftime`,
        and used to parse date strings into date object.

        `ValueError` is raised if the date_string and format can’t be
        parsed by time.strptime() or if it returns a value which isn’t a time tuple. For a
        complete list of formatting directives, see :doc:`/directives`.


        :param date_string:
        :param format:
        :return: A :py:class:`khayyam.JalaliDate` corresponding to date_string, parsed according to format
        :rtype: :py:class:`khayyam.JalaiDate`
        """
        result = cls.formatterfactory(format).parse(date_string)
        result = {k:v for k, v in result.items() if k in ('year', 'month', 'day')}
        return cls(**result)

    @staticmethod
    def _validate(year, month, day):
        year = year if isinstance(year, int) else int(year)
        month = month if isinstance(month, int) else int(month)
        day = day if isinstance(day, int) else int(day)

        if year < MINYEAR or year > MAXYEAR:
            raise ValueError('Year must be between %s and %s, but it is: %s' % (MINYEAR, MAXYEAR, year))
        if month < 1 or month > 12:
            raise ValueError('Month must be between 1 and 12, but it is: %s' % month)
        _days_in_month = days_in_month(year, month)
        if day < 1 or day > _days_in_month:
            raise ValueError('Day must be between 1 and %s, but it is: %s' % (_days_in_month, day))
        return year, month, day

    def tojulianday(self):
        """
        :return: Julian day representing the current instance.
        :rtype: int
        """
        return julian_day_from_jalali_date(self.year, self.month, self.day)

    def copy(self):
        """
        It's equivalent to:

            >>> source_date = JalaliDate(1394, 3, 24)
            >>> JalaliDate(source_date.year, source_date.month, source_date.day)
            khayyam.JalaliDate(1394, 3, 24, Yekshanbeh)

        :return: A Copy of the current instance.
        :rtype: :py:class:`khayyam.JalaiDate`
        """
        return JalaliDate(self.year, self.month, self.day)

    def replace(self, year=None, month=None, day=None):
        """
        Replaces the given arguments on this instance, and return a new instance.

        :param year:
        :param month:
        :param day:
        :return: A :py:class:`khayyam.JalaliDate` with the same attributes, except for those
            attributes given new values by which keyword arguments are specified.
        """

        return JalaliDate(
            year if year else self.year,
            month if month else self.month,
            day if day else self.day
        )

    def todate(self):
        """
        Calculates the corresponding day in the gregorian calendar. this is the main use case of this library.

        :return: Corresponding date in gregorian calendar.
        :rtype: :py:class:`datetime.date`
        """
        arr = gregorian_date_from_julian_day(self.tojulianday())
        return datetime.date(int(arr[0]), int(arr[1]), int(arr[2]))

    def toordinal(self):
        """
        It's equivalent to:
        .. testsetup:: api-toordinal

            import khayyam
            from khayyam import JalaliDate

        .. doctestLL api-toordinal

            >>> d = JalaliDate(1361, 6, 15)
            >>> (d - JalaliDate(khayyam.MINYEAR)).days + 1

        :return: The corresponding proleptic Shamsi ordinal days.
        :rtype: int
        """
        return (self - self.min).days + 1

    def timetuple(self):
        """
        It's equivalent to:

            >>> time.struct_time((d.year, d.month, d.day, d.hour, d.minute, d.second, d.weekday(), dayofyear, [-1|1|0])) # doctest: +SKIP
            time.struct_time(tm_year=2015, tm_mon=7, tm_mday=28, tm_hour=0, tm_min=0, tm_sec=0, tm_wday=1, tm_yday=209, tm_isdst=-1)


        The tm_isdst flag of the result is set according to the dst() method: `tzinfo`
        is None or dst() returns None, tm_isdst is set to -1; else if dst()
        returns a non-zero value, tm_isdst is set to 1; else tm_isdst is set to 0.


        :return: A :py:class:`time.struct_time` such as returned by time.localtime().
        :rtype: :py:class:`time.struct_time`
        """
        return time.struct_time((
            self.year,
            self.month,
            self.day,
            0,
            0,
            0,
            self.weekday(),
            self.dayofyear(),
            -1
        ))

    def weekday(self):
        """
        :rtype: int
        :return: The day of the week as an integer, where Saturday is 0 and Friday is 6.
        """
        return (self.todate().weekday() + 2) % 7

    def isoweekday(self):
        """
        :rtype: int
        :return: The day of the week as an integer, where Saturday is 1 and Friday is 7.
        """
        return self.weekday() + 1

    def isocalendar(self):
        """
        :rtype: tuple
        :return: Return a 3-tuple, (year, week number, isoweekday).
        """
        return self.year, self.weekofyear(SATURDAY), self.isoweekday()

    def isoformat(self):
        """
        :rtype: str
        :return: A string representing the date in ISO 8601 format, ‘YYYY-MM-DD’.
        For example:

            >>> JalaliDate(1361, 12, 4).isoformat() == '1361-12-04'
            True

        """
        return self.strftime('%Y-%m-%d')

    def strftime(self, format_string):
        """
        Format codes referring to hours, minutes or seconds will see 0 values.
        For a complete list of formatting directives, see :doc:`/directives`.

        :param format_string: The format string.
        :return: A string representing the date, controlled by an explicit format string
        :rtype: unicode
        """
        return self.formatterfactory(format_string).format(self)

    def weekdayname(self):
        """
        :return: The corresponding persian weekday name: [شنبه - جمعه]
        :rtype: unicode
        """
        return PERSIAN_WEEKDAY_NAMES[self.weekday()]

    def weekdayabbr(self):
        """
        :return: The corresponding persian weekday abbreviation: [ش ی د س چ پ ج]
        :rtype: unicode
        """
        return PERSIAN_WEEKDAY_ABBRS[self.weekday()]

    def weekdaynameascii(self):
        """
        :rtype: unicode
        :return: The corresponding persian weekday name in ASCII:
                [Shanbeh - Jomeh]
        """
        return PERSIAN_WEEKDAY_NAMES_ASCII[self.weekday()]

    def weekdayabbrascii(self):
        """
        :return: The corresponding persian weekday abbreviation in ASCII:
            [Sh, Y, D, Se, Ch, P, J]
        :rtype: unicode
        """
        return PERSIAN_WEEKDAY_ABBRS_ASCII[self.weekday()]

    def monthname(self):
        """
        :rtype: unicode
        :return: The corresponding persian month name: [فروردین - اسفند]
        """
        return PERSIAN_MONTH_NAMES[self.month]

    def monthabbr(self):
        """
        :rtype: unicode
        :return: The corresponding persian month abbreviation:
                [فر, ار, خر, تی, مر, شه, مه, آب, آذ, دی, به, اس]
        """
        return PERSIAN_MONTH_ABBRS[self.month]

    def monthabbr_ascii(self):
        """
        :rtype: unicode
        :return: The corresponding persian month abbreviation in ASCII: [F, O , Kh ... E].
        """
        return PERSIAN_MONTH_ABBRS_ASCII[self.month]

    def monthnameascii(self):
        """
        :rtype: unicode
        :return: The corresponding persian month name in ASCII:
            [Farvardin - Esfand]
        """
        return PERSIAN_MONTH_NAMES_ASCII[self.month]

    def localdateformat(self):
        """
        It's equivalent to:

        .. testsetup:: api-localdateformat

            from __future__ import print_function
            from khayyam import JalaliDate

        .. doctest:: api-localdateformat

            >>> print(JalaliDate(1361, 6, 15).strftime('%A %D %B %N'))
            دوشنبه ۱۵ شهریور ۱۳۶۱

        For example:

        .. doctest:: api-localdateformat

            >>> print(JalaliDate(1394, 5, 6).localdateformat())
            سه شنبه ۶ مرداد ۱۳۹۴


        :return: Appropriate localized string representing a persian day
        :rtype: unicode
        """
        return self.strftime('%A %D %B %N')

    def firstdayofyear(self):
        """
        As it's name says: it's referring to a :py:class:`JalaliDate`
        representing the first day of current instance's year.

        :return: First day of corresponding year.
        :rtype: :py:class:`JalaliDate`
        """
        return JalaliDate(self.year, 1, 1)

    def dayofyear(self):
        """
        :return: Day of year az integer: 1-35[5,6]
        :rtype: int
        """
        return (self - self.firstdayofyear()).days + 1

    def weekofyear(self, first_day_of_week=SATURDAY):
        """weekofyear(first_day_of_week=SATURDAY)

        :param first_day_of_week: One of the
                :py:data:`khayyam.SATURDAY`,
                :py:data:`khayyam.SUNDAY`,
                :py:data:`khayyam.MONDAY`,
                :py:data:`khayyam.TUESDAY`,
                :py:data:`khayyam.WEDNESDAY`,
                :py:data:`khayyam.THURSDAY` or
                :py:data:`khayyam.FRIDAY`
        :return: The week number of the year.
        :rtype: int
        """
        first_day_of_year = self.firstdayofyear()
        days = (self - first_day_of_year).days
        offset = first_day_of_week - first_day_of_year.weekday()
        if offset < 0:
            offset += 7

        if days < offset:
            return 0

        return int((days - offset) / 7 + 1)

    ###################
    # Special Members #
    ###################

    __format__ = strftime

    def __str__(self):
        return self.isoformat()

    @force_encoded_string_output
    def __repr__(self):
        return 'khayyam.JalaliDate(%s, %s, %s, %s)' % \
               (self.year, self.month, self.day, self.weekdaynameascii())

    def __add__(self, x):
        if isinstance(x, datetime.timedelta):
            days = self.tojulianday() + x.days
            return JalaliDate(julian_day=days)

        raise ValueError('JalaliDate object can added by timedelta or JalaliDate object')

    def __sub__(self, x):
        if isinstance(x, datetime.timedelta):
            days = self.tojulianday() - x.days
            return JalaliDate(julian_day=days)
        elif isinstance(x, JalaliDate):
            days = self.tojulianday() - x.tojulianday()
            return datetime.timedelta(days=days)

        raise ValueError('JalaliDate object can added by timedelta or JalaliDate object')

    def __lt__(self, x):
        assert isinstance(x, JalaliDate), 'Comparison just allow with JalaliDate'
        return self.tojulianday() < x.tojulianday()

    def __le__(self, x):
        assert isinstance(x, JalaliDate), 'Comparison just allow with JalaliDate'
        return self.tojulianday() <= x.tojulianday()

    def __hash__(self):
        return hash(self.year) ^ hash(self.month) ^ hash(self.day)

    def __eq__(self, x):
        if not x:
            return False
        if isinstance(x, datetime.date):
            return self.todate().__eq__(x)
        elif isinstance(x, JalaliDate):
            return hash(self) == hash(x)
        else:
            raise ValueError('Comparison only allowed with JalaliDate and datetime.date objects.')

    def __ne__(self, x):
        return not self.__eq__(x)

    def __gt__(self, x):
        assert isinstance(x, JalaliDate), 'Comparison just allow with JalaliDate'
        return self.tojulianday() > x.tojulianday()

    def __ge__(self, x):
        assert isinstance(x, JalaliDate), 'Comparison just allow with JalaliDate'
        return self.tojulianday() >= x.tojulianday()


# Class attributes
JalaliDate.min = JalaliDate(*JalaliDate.min)
JalaliDate.max = JalaliDate(*JalaliDate.max)




# -*- coding: utf-8 -*-
import time
import unittest
from khayyam import JalaliDate, MAXYEAR
from datetime import timedelta, date
__author__ = 'vahid'


class TestJalaliDate(unittest.TestCase):
    
    def setUp(self):
        self.leap_year = 1375
    
    def test_instantiate(self):
        
        jdate = JalaliDate(1376, 5, 23)
        self.assertFalse(jdate is None)

        self.assertEqual(JalaliDate(jdate.todate()), jdate)
        self.assertEqual(JalaliDate(jdate), jdate)

        self.assertEqual(JalaliDate(julian_day=2450674), jdate)

        self.assertRaises(ValueError, JalaliDate, MAXYEAR + 1, 5, 23)
        self.assertRaises(ValueError, JalaliDate, MAXYEAR, 13, 23)
        self.assertRaises(ValueError, JalaliDate, MAXYEAR, 12, 30)

    def test_repr(self):
        jdate = JalaliDate(1376, 5, 23)
        self.assertEqual(repr(jdate), 'khayyam.JalaliDate(1376, 5, 23, Panjshanbeh)')

    def test_is_leap(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        self.assertTrue(jdate.isleap)
        
    def test_days_in_month(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        self.assertEqual(jdate.daysinmonth, 30)
        
    def test_to_from_julian_day(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        jdate2 = JalaliDate(julian_day=jdate.tojulianday())
        self.assertEqual(jdate, jdate2)
        
    def test_to_from_date(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        jdate2 = JalaliDate(jdate.todate())
        self.assertEqual(jdate, jdate2)

    def test_iso_calendar(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        self.assertEqual(jdate.isocalendar(), (self.leap_year, 51, 6))
        
    def test_iso_format(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        self.assertEqual(jdate.isoformat(), '%s-12-23' % self.leap_year)

    def test_strftime(self):
        jdate = JalaliDate.strptime(JalaliDate(self.leap_year, 12, 23).isoformat(), '%Y-%m-%d')
        self.assertEqual(jdate.isoformat(), '%s-12-23' % self.leap_year)
        d1 = JalaliDate(1361, 6, 15)
        self.assertEqual(d1.strftime('%Y-%m-%d'), '1361-06-15')
        self.assertEqual(d1.strftime(u'اول%Y-%m-%dآخر'), u'اول1361-06-15آخر')

    def test_add(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        jdate2 = jdate + timedelta(10)
        
        self.assertEqual(jdate2, JalaliDate(self.leap_year + 1, 1, 3))
        
    def test_sub(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        jdate2 = jdate - timedelta(10)
        
        self.assertEqual(jdate2, JalaliDate(self.leap_year, 12, 13))
        
        jtimedelta = jdate - JalaliDate(self.leap_year - 1, 12, 1)
        
        self.assertEqual(jtimedelta, timedelta(387))
        
    def test_lt_gt_le_ge_ne_eg(self):
        jdate = JalaliDate(self.leap_year, 12, 23)
        jdate2 = JalaliDate(self.leap_year, 12, 24)
        jdate3 = JalaliDate(self.leap_year, 12, 24)
        
        self.assertTrue(jdate <= jdate2)
        self.assertTrue(jdate != jdate2)
        self.assertFalse(jdate > jdate2)
        self.assertTrue(jdate2 == jdate3)
    
    def test_ordinal(self):
        min = JalaliDate.fromordinal(1)
        max = JalaliDate.fromordinal(JalaliDate.max.toordinal())
        self.assertEqual(min.year, 1)
        self.assertEqual(min.month, 1)
        self.assertEqual(min.day, 1)
        self.assertEqual(min, JalaliDate.min)
        self.assertEqual(max, JalaliDate.max)

    def test_algorithm(self):
        min = date(623, 1, 1)
        max_days = 5000
        days = 0
        while True:
            dt = min + timedelta(days=days)
            jd = JalaliDate(dt)
            # print('Processing day: %s' % jd)
            dt2 = jd.todate()
            self.assertEqual(dt, dt2)
            days += 1
            if days > max_days:
                break

    def test_replace(self):
        d1 = JalaliDate(1391, 12, 30)
        self.assertEqual(d1.replace(year=1395), JalaliDate(1395, 12, 30))
        self.assertEqual(d1.replace(month=1),   JalaliDate(1391, 1, 30))
        self.assertEqual(d1.replace(day=1),     JalaliDate(1391, 12, 1))
        self.assertRaises(ValueError, d1.replace, year=1392)

    def test_strptime(self):
        """
        %Z not working at all
        """
        self.assertEqual(JalaliDate.strptime('1361', '%Y'), JalaliDate(1361))
        self.assertEqual(JalaliDate.strptime('1361%C', '%Y%C'), JalaliDate(1361))
        self.assertEqual(JalaliDate.strptime(u'اریا1361گلگشت', u'اریا%Yگلگشت'), JalaliDate(1361))

        current_century = int(JalaliDate.today().year / 100) * 100
        self.assertEqual(JalaliDate.strptime('61', '%y'), JalaliDate(current_century + 61))
        self.assertEqual(JalaliDate.strptime('61%C', '%y%C'), JalaliDate(current_century + 61))
        self.assertEqual(JalaliDate.strptime(u'اریا61گلگشت', u'اریا%yگلگشت'), JalaliDate(current_century+61))
        self.assertEqual(JalaliDate.strptime(u'جمعه 01 اردیبهشت 0001', u'%A %d %B %Y'),
                         JalaliDate(month=2, day=1))

    def test_timetuple(self):
        self.assertEqual(
            JalaliDate(1361, 6, 15).timetuple(),
            time.struct_time((1361, 6, 15, 0, 0, 0, 2, 170, -1)))

if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-
from .directive import Directive
from khayyam.formatting import constants as consts
from khayyam.compat import get_unicode
__author__ = 'vahid'


class BaseAmPmDirective(Directive):
    def __init__(self, key, name, regex):
        super(BaseAmPmDirective, self).__init__(key, name, regex, get_unicode)

    def format(self, d):
        return getattr(d, self.name)()

    def is_am(self, ctx):
        raise NotImplementedError()

    def post_parser(self, ctx, formatter):
        hour12 = ctx['hour12']

        if self.is_am(ctx): # AM
            hour = 0 if hour12 == 12 else hour12
        else: # PM
            hour = hour12 + (12 if hour12 < 12 else 0)

        ctx['hour'] = hour


class AmPmDirective(BaseAmPmDirective):

    def is_am(self, ctx):
        return ctx['ampm'] == consts.AM_PM[0]


class AmPmASCIIDirective(BaseAmPmDirective):

    def is_am(self, ctx):
        return ctx[self.name] == consts.AM_PM_ASCII[0]




# -*- coding: utf-8 -*-
import unittest
from datetime import timedelta
from khayyam import JalaliDate
from khayyam.formatting import constants as c
from rtl import rtl
__author__ = 'vahid'


class JalaliDateFormatterTestCase(unittest.TestCase):

    def assert_parse_and_format(self, jdate ,fmt, print_=False):
        jdate_str = jdate.strftime(fmt)
        if print_:
            print(jdate_str)
        d2 = JalaliDate.strptime(jdate_str, fmt)
        self.assertEqual(jdate, d2)


    def test_week(self):
        """
        Testing:
            %a           Locale’s abbreviated weekday name.
            %A           Locale’s full weekday name.
            %e           ASCII Locale’s abbreviated weekday name.
            %E           ASCII Locale’s full weekday name.
            %w           Weekday as a decimal number [0(Saturday), 6(Friday)].
            %W           Week number of the year (SATURDAY as the first day of the week) as a decimal number [00, 53]. All days in a new year preceding the first Monday are considered to be in week 0.
            %U           Week number of the year (Sunday as the first day of the week) as a decimal number [00, 53]. All days in a new year preceding the first Sunday are considered to be in week 0.

        """

        d1 = JalaliDate(1361, 6, 15)
        self.assertEqual(d1.strftime('%a'), u'د')
        self.assertEqual(d1.strftime('%A'), u'دوشنبه')
        self.assertEqual(d1.strftime('%e'), u'D')
        self.assertEqual(d1.strftime('%E'), u'Doshanbeh')
        self.assertEqual(d1.strftime('%w'), u'2')
        self.assertEqual(d1.strftime('%W'), u'24')


        for i in range(7):
            self.assert_parse_and_format(JalaliDate.min + timedelta(i), '%d %w %W %U')
            self.assertEqual(
                JalaliDate.strptime('1345 10 10 %s' % c.PERSIAN_WEEKDAY_ABBRS[i], '%Y %m %d %a'),
                JalaliDate(year=1345, month=10, day=10))
            self.assertEqual(
                JalaliDate.strptime('1345 10 10 %s' % c.PERSIAN_WEEKDAY_ABBRS_ASCII[i], '%Y %m %d %e'),
                JalaliDate(year=1345, month=10, day=10))
            self.assertEqual(
                JalaliDate.strptime('1345 10 10 %s' % c.PERSIAN_WEEKDAY_NAMES[i], '%Y %m %d %A'),
                JalaliDate(year=1345, month=10, day=10))
            self.assertEqual(
                JalaliDate.strptime('1345 10 10 %s' % c.PERSIAN_WEEKDAY_NAMES_ASCII[i], '%Y %m %d %E'),
                JalaliDate(year=1345, month=10, day=10))

    def test_year(self):
        """
        Testing:
            %y           Year without century as a zero padded decimal number [00, 99].
            %n           Year without century as a decimal number in persian form [۱, ۹۹].
            %u           Year without century as a zero padded decimal number in persian form [۰۱, ۹۹].
            %Y           Year with century as a decimal number [1-3178].
            %N           Year with century as a decimal number in persian form [۱-۳۱۷۸].
            %O           Year with century as a zero padded decimal number in persian form [۰۰۰۱-۳۱۷۸].
        """

        self.assertEqual(JalaliDate(1361, 6, 15).strftime('%y'), u'61')
        self.assertEqual(JalaliDate(1361, 6, 15).strftime('%Y'), u'1361')
        self.assertEqual(JalaliDate(61, 11, 5).strftime('%N'), u'۶۱')
        self.assertEqual(JalaliDate(61, 11, 5).strftime('%O'), u'۰۰۶۱')

        self.assertEqual(JalaliDate.strptime('94', '%y'), JalaliDate(1394))
        self.assertEqual(JalaliDate.strptime('01', '%y'), JalaliDate(1301))
        self.assertEqual(JalaliDate.strptime('00', '%y'), JalaliDate(1300))
        self.assertEqual(JalaliDate.strptime(u'۹۴', '%n'), JalaliDate(1394))
        self.assertEqual(JalaliDate.strptime(u'۱', '%n'), JalaliDate(1301))
        self.assertEqual(JalaliDate.strptime(u'۰', '%n'), JalaliDate(1300))
        self.assertEqual(JalaliDate.strptime(u'۹۴', '%u'), JalaliDate(1394))
        self.assertEqual(JalaliDate.strptime(u'۰۱', '%u'), JalaliDate(1301))
        self.assertEqual(JalaliDate.strptime(u'۰۰', '%u'), JalaliDate(1300))

        self.assertEqual(JalaliDate.strptime(u'۰۰۴', '%N'), JalaliDate(4, 1, 1))
        self.assertEqual(JalaliDate.strptime(u'۰۴', '%N'), JalaliDate(4, 1, 1))
        self.assertEqual(JalaliDate.strptime(u'۴', '%N'), JalaliDate(4, 1, 1))

        self.assertEqual(JalaliDate.strptime(u'۰۰۴', '%O'), JalaliDate(4, 1, 1))
        self.assertEqual(JalaliDate.strptime(u'۰۴', '%O'), JalaliDate(4, 1, 1))
        self.assertEqual(JalaliDate.strptime(u'۴', '%O'), JalaliDate(4, 1, 1))


        this_century = JalaliDate(int(JalaliDate.today().year/100) * 100)
        for i in range(99):
            self.assert_parse_and_format(this_century.replace(year=this_century.year+i), '%y')
            self.assert_parse_and_format(this_century.replace(year=this_century.year+i), '%n')
            self.assert_parse_and_format(this_century.replace(year=this_century.year+i), '%u')

        for i in range(1, 1001):
            self.assert_parse_and_format(JalaliDate(i), '%Y')
            self.assert_parse_and_format(JalaliDate(i), '%N')
            self.assert_parse_and_format(JalaliDate(i), '%O')


    def test_month(self):
        """
        Testing:
            %m           Month as a decimal number [01, 12].
            %R           Month as a decimal number in persian form [۱, ۱۲].
            %P           Month as a zero padded decimal number in persian form [۰۱, ۱۲].
            %b           Locale’s abbreviated month name.
            %B           Locale’s full month name.
            %g           ASCII Locale’s abbreviated month name.
            %G           ASCII Locale’s full month name.

        """

        d1 = JalaliDate(1361, 6, 15)
        self.assertEqual(d1.strftime('%b'), u'شه')
        self.assertEqual(d1.strftime('%B'), u'شهریور')
        self.assertEqual(d1.strftime('%m'), u'06')
        self.assertEqual(d1.strftime('%g'), u'Sh')
        self.assertEqual(d1.strftime('%G'), u'Shahrivar')

        self.assertEqual(JalaliDate(1361, 1, 5).strftime('%R'), u'۱')
        self.assertEqual(JalaliDate(1361, 11, 5).strftime('%P'), u'۱۱')
        self.assertEqual(JalaliDate(1361, 1, 5).strftime('%P'), u'۰۱')

        self.assertEqual(JalaliDate.strptime(u'۷', '%R'), JalaliDate(1, 7, 1))
        self.assertRaises(ValueError, JalaliDate.strptime, u'۰۷', '%R')
        self.assertEqual(JalaliDate.strptime(u'۰۷', '%P'), JalaliDate(1, 7, 1))
        self.assertRaises(ValueError, JalaliDate.strptime, u'۷', '%P')


        # Test months
        for i in range(1, 13):
            self.assertEqual(JalaliDate.strptime(str(i), '%m'), JalaliDate(month=i))
            self.assertEqual(JalaliDate.strptime('1345 %s' % c.PERSIAN_MONTH_ABBRS[i], '%Y %b'),
                             JalaliDate(year=1345, month=i, day=1))
            self.assertEqual(JalaliDate.strptime('1345 %s' % c.PERSIAN_MONTH_NAMES[i], '%Y %B'),
                             JalaliDate(year=1345, month=i, day=1))
            self.assertEqual(JalaliDate.strptime('1345 %s' % c.PERSIAN_MONTH_ABBRS_ASCII[i], '%Y %g'),
                             JalaliDate(year=1345, month=i, day=1))
            self.assertEqual(JalaliDate.strptime('1345 %s' % c.PERSIAN_MONTH_NAMES_ASCII[i], '%Y %G'),
                             JalaliDate(year=1345, month=i, day=1))
            self.assertEqual(JalaliDate.strptime('1345 %s' % rtl(str(i), digits=True), '%Y %R'),
                             JalaliDate(year=1345, month=i, day=1))
            self.assertEqual(JalaliDate.strptime('1345 %s' % rtl('%.2d' % i, digits=True), '%Y %P'),
                             JalaliDate(year=1345, month=i, day=1))



        self.assertRaises(ValueError, JalaliDate.strptime, '13', '%m')
        self.assertRaises(ValueError, JalaliDate.strptime, '0', '%m')
        self.assertRaises(ValueError, JalaliDate.strptime, u'1345 مت', '%Y %b')
        self.assertRaises(ValueError, JalaliDate.strptime, u'1345 شتران', '%Y %B')
        self.assertRaises(ValueError, JalaliDate.strptime, u'1345 مت', '%Y %g')
        self.assertRaises(ValueError, JalaliDate.strptime, u'1345 شتران', '%Y %G')


        start_date = JalaliDate()
        for i in range(1, 501):
            self.assert_parse_and_format(start_date + timedelta(days=i) , '%Y %m %d')
            self.assert_parse_and_format(start_date + timedelta(days=i) , '%Y %b %d')
            self.assert_parse_and_format(start_date + timedelta(days=i) , '%Y %B %d')
            self.assert_parse_and_format(start_date + timedelta(days=i) , '%Y %g %d')
            self.assert_parse_and_format(start_date + timedelta(days=i) , '%Y %G %d')
            self.assert_parse_and_format(start_date + timedelta(days=i) , '%Y %R %d')
            self.assert_parse_and_format(start_date + timedelta(days=i) , '%Y %P %d')


    def test_day(self):
        """
        Testing:
            %d           Day of the month as a decimal number [01, 31].
            %D           Day of the month as a decimal number in persian form [۱, ۳۱].
            %K           Day of the month as a zero padded decimal number in persian form [۰۱, ۳۱].
            %j           Day of the year as a zero padded decimal number [001, 366].
            %J           Day of the year as a decimal number in persian form [۱, ۳۶۶].
            %V           Day of the year as a zero padded decimal number in persian form [..۱, ۳۶۶].

        """

        d1 = JalaliDate(1361, 6, 15)
        self.assertEqual(d1.strftime('%d'), u'15')
        self.assertEqual(d1.strftime('%j'), u'170')
        self.assertEqual(d1.strftime('%D'), u'۱۵')

        self.assertEqual(JalaliDate(1361, 6, 2).strftime('%D'), u'۲')
        self.assertEqual(JalaliDate(1361, 6, 2).strftime('%K'), u'۰۲')
        self.assertEqual(JalaliDate(1361, 6, 2).strftime('%J'), u'۱۵۷')
        self.assertEqual(JalaliDate(1361, 1, 5).strftime('%J'), u'۵')
        self.assertEqual(JalaliDate(1361, 1, 25).strftime('%V'), u'۰۲۵')
        self.assertEqual(JalaliDate(1361, 1, 5).strftime('%V'), u'۰۰۵')

        # Test days
        for i in range(1, 32):
            self.assertEqual(JalaliDate.strptime(str(i), '%d'), JalaliDate(day=i))

        self.assertRaises(ValueError, JalaliDate.strptime, '32', '%d')
        self.assertRaises(ValueError, JalaliDate.strptime, '0', '%d')

        # Test day of year
        for i in range(1, 366):
            self.assertEqual(JalaliDate.strptime(str(i), '%j'), JalaliDate.fromordinal(i))
        self.assertRaises(ValueError, JalaliDate.strptime, '366', '%j')
        self.assertRaises(ValueError, JalaliDate.strptime, '0', '%j')
        self.assertEqual(JalaliDate.strptime('1345 5', '%Y %j'),
                         JalaliDate(year=1345, month=1, day=5))

        self.assertEqual(JalaliDate.strptime('1302 123 3 4', '%Y %j %m %d'),
                         JalaliDate(year=1302, month=4, day=30))

        self.assertEqual(JalaliDate.strptime('1302 3 4', '%Y %m %d'),
                         JalaliDate(year=1302, month=3, day=4))

        self.assertEqual(JalaliDate.strptime(u'۲', '%D'), JalaliDate(2))
        self.assertRaises(ValueError, JalaliDate.strptime, u'۰۲', '%D')

        self.assertEqual(JalaliDate.strptime(u'۰۲', '%K'), JalaliDate(1, 1, 2))
        self.assertRaises(ValueError, JalaliDate.strptime, u'۲', '%K')

        self.assertEqual(JalaliDate.strptime(u'۰۲', '%K'), JalaliDate(1, 1, 2))
        self.assertRaises(ValueError, JalaliDate.strptime, u'۲', '%K')

        self.assertEqual(JalaliDate.strptime(u'۱۵۷', '%J'), JalaliDate(1, 6, 2))
        self.assertEqual(JalaliDate.strptime(u'۷', '%J'), JalaliDate(1, 1, 7))
        self.assertRaises(ValueError, JalaliDate.strptime, u'۰۷', '%J')
        self.assertRaises(ValueError, JalaliDate.strptime, u'۰۴۷', '%J')

        self.assertEqual(JalaliDate.strptime(u'۰۴۷', '%V'), JalaliDate(1, 2, 16))
        self.assertEqual(JalaliDate.strptime(u'۰۰۷', '%V'), JalaliDate(1, 1, 7))
        self.assertRaises(ValueError, JalaliDate.strptime, u'۷', '%V')
        self.assertRaises(ValueError, JalaliDate.strptime, u'۴۷', '%V')


        for i in range(1, 400):
            self.assert_parse_and_format(JalaliDate.fromordinal(i), "%Y-%m-%d")
            self.assert_parse_and_format(JalaliDate.fromordinal(i), "%Y-%m-%D")
            self.assert_parse_and_format(JalaliDate.fromordinal(i), "%Y-%m-%K")
            self.assert_parse_and_format(JalaliDate.fromordinal(i), "%Y-%j")
            self.assert_parse_and_format(JalaliDate.fromordinal(i), "%Y-%J")
            self.assert_parse_and_format(JalaliDate.fromordinal(i), "%Y-%V")


    def test_locale_date(self):
        """
        Testing:
            %x           Locale’s appropriate date representation.
        """
        self.assertEqual(JalaliDate(1361, 6, 15).strftime('%x'), u'دوشنبه ۱۵ شهریور ۱۳۶۱')
        self.assertEqual(JalaliDate().strftime('%x'), u'جمعه ۱ فروردین ۱')
        self.assertEqual(JalaliDate.strptime(u'جمعه ۱ فروردین ۱', '%x'), JalaliDate.min)

        self.assertEqual(JalaliDate(1375, 1, 31).strftime('%x'), u'جمعه ۳۱ فروردین ۱۳۷۵')
        self.assertEqual(JalaliDate.strptime(u'جمعه ۳۱ فروردین ۱۳۷۵%', '%x%%'),
                         JalaliDate(1375, 1, 31))


    def test_percent(self):
        """
        Testing:
            %%      A literal '%' character.
        """
        self.assert_parse_and_format(JalaliDate(1375, 1, 31), "%Y-%m-%d %%")
        self.assert_parse_and_format(JalaliDate(1375, 1, 31), "%Y-%m-%d %% %% %%")


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

__version__ = '2.9.8a'

#: Minimum year supported by the library.
MINYEAR = 1

#: Maximum year supported by the library.
MAXYEAR = 3178

#: Representing the Saturday weekday.
SATURDAY = 0

#: Representing the Sunday weekday.
SUNDAY = 1

#: Representing the Monday weekday.
MONDAY = 2

#: Representing the Tuesday weekday.
TUESDAY = 3


#: Representing the Wednesday weekday.
WEDNESDAY = 4


#: Representing the Thursday weekday.
THURSDAY = 5


#: Representing the Friday weekday.
FRIDAY = 6


from .jalali_date import JalaliDate
from .jalali_datetime import JalaliDatetime
from .timezones import TehranTimezone, Timezone
teh_tz = TehranTimezone()
__author__ = 'vahid'

__all__ = [
    'MINYEAR',
    'MAXYEAR',
    'SATURDAY',
    'SUNDAY',
    'MONDAY',
    'THURSDAY',
    'WEDNESDAY',
    'TUESDAY',
    'FRIDAY',
    'JalaliDate',
    'JalaliDatetime',
    'TehranTimezone',
    'Timezone',
    'teh_tz'
]



# -*- coding: utf-8 -*-
from .persian import PersianNumberDirective
__author__ = 'vahid'


class PersianMonthDirective(PersianNumberDirective):

    def format(self, d):
        return super(PersianMonthDirective, self).format(d.month)

    def post_parser(self, ctx, formatter):
        super(PersianMonthDirective, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['month'] = ctx[self.name]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import sys
import re
from .compat import get_unicode
import warnings
__author__ = 'vahid'


def force_encoded_string_output(func):
    if sys.version_info.major < 3:
        def _func(*args, **kwargs):
            return func(*args, **kwargs).encode('utf-8')
        return _func
    else:
        return func


def deprecated(func):
    """This is a decorator which can be used to mark functions
    as deprecated. It will result in a warning being emmitted
    when the function is used."""
    def new_func(*args, **kwargs):
        warnings.warn("Call to deprecated function %s." % func.__name__,
                      category=DeprecationWarning)
        return func(*args, **kwargs)
    new_func.__name__ = func.__name__
    new_func.__doc__ = func.__doc__
    new_func.__dict__.update(func.__dict__)
    return new_func


@deprecated
def replace_if_match(data, pattern, new):
    if re.search(pattern, data):
        if hasattr(new, '__call__'):
            new = new()
        return data.replace(pattern, get_unicode(new))
    return data





from libc.math cimport floor, ceil
from cpython cimport bool

cpdef double get_julian_day_from_gregorian(int year, int month, int day):
    cdef:
        double _year = year
        double _month = month
        double _day = day
        double _century = 0
        double result = 0

    if year / 4.0 == round(year / 4.0):
        if year / 100.0 == round(year / 100.0):
            if year / 400.0 == round(year / 400.0):
                # Leap year checking #
                if month == 2:
                    assert day <= 29, 'Invalid date'
        else:
            # Leap year #
            if month == 2:
                assert day <= 29, 'Invalid date'


    if _month <= 2:
        _year -= 1
        _month += 12

    _century = floor(_year / 100)
    result += floor(365.25 * (_year + 4716))
    result += floor(30.6001 * (_month + 1))
    result += _day
    result += 2 - _century + floor(_century / 4)
    result -= 1524.5

    return result


cpdef bool is_leap_year(int year):
    return ((((((year - [473, 474][year > 0]) % 2820) + 474) + 38) * 682) % 2816) < 682


cpdef int days_in_year(int year):
    return 366 if is_leap_year(year) else 365


cpdef int days_in_month(int year, int month):
    if 1 <= month <= 6:
        return 31
    elif 7 <= month < 12:
        return 30

    assert month == 12, 'Month must be between 1 and 12'

    # Esfand(اسفند)
    if is_leap_year(year):
        return 30
    else:
        return 29

cpdef double julian_day_from_jalali_date(int year, int month, int day):
    cdef:
        int base = year - (474 if year >= 0 else 473)
        int julian_year = 474 + (base % 2820)
        double result = day

    result += [((month - 1) * 30) + 6, (month - 1) * 31][month <= 7]
    result += floor(((julian_year * 682) - 110) / 2816)
    result += (julian_year - 1) * 365
    result += floor(base / 2820) * 1029983
    result += 1948320.5 - 1
    return result


cpdef tuple jalali_date_from_julian_day(double jd):
    cdef:
        double julian_day = floor(jd) + 0.5
        double offset = julian_day - 2121445.5 # julian_day_from_jalali(475, 1, 1) replaced by its static value
        double cycle = floor(offset / 1029983)
        double remaining = offset % 1029983
        double a1, a2, year_cycle, days_in_year
        int year, month, day

    if remaining == 1029982:
        year_cycle = 2820
    else:
        a1 = floor(remaining / 366)
        a2 = remaining % 366
        year_cycle = floor(((2134 * a1) + (2816 * a2) + 2815) / 1028522) + a1 + 1
    year = <int>(year_cycle + (2820 * cycle) + 474)
    if year <= 0:
        year -= 1
    days_in_year = (julian_day - julian_day_from_jalali_date(year, 1, 1)) + 1
    month = <int>ceil([(days_in_year - 6) / 30, days_in_year / 31][days_in_year <= 186])
    day = <int>(julian_day - julian_day_from_jalali_date(year, month, 1)) + 1
    return year, month, day


cpdef tuple gregorian_date_from_julian_day(double jd):
    cdef:
        double year, month, day
        double julian_day, actual_days, remains, alpha, b, c, d, e

    if jd <= 0:
        raise ValueError('Invalid Date')

    julian_day = jd + 0.5
    actual_days = floor(julian_day)
    remains = julian_day - actual_days

    alpha = floor((actual_days - 1867216.25) / 36524.25)
    b = (actual_days + 1 + alpha - floor(alpha / 4)) + 1524
    c = floor((b - 122.1) / 365.25)
    d = floor(365.25 * c)
    e = floor((b - d) / 30.6001)
    day = b - d - floor(30.6001 * e) + remains

    if e < 14:
        month = e - 1
    elif e == 14 or e == 15:
        month = e - 13
    else:
        raise ValueError('Cannot calculate month')


    if month > 2:
        year = c - 4716
    elif month == 1 or month == 2:
        year = c - 4715
    else:
        raise ValueError('Invalid month: %s' % month)

    return <int>year, <int>month, <int>day

cpdef tuple jalali_date_from_gregorian_date(int year, int month, int day):
    return jalali_date_from_julian_day(get_julian_day_from_gregorian(year, month, day))



# -*- coding: utf-8 -*-
import unittest
from khayyam import algorithms_c as alg_c
from khayyam import algorithms_pure as alg_p
__author__ = 'vahid'

class TestCAlgorithms(unittest.TestCase):

    def test_get_julian_day_from_gregorian(self):
        for i in range(3000):
            self.assertEqual(
                alg_c.get_julian_day_from_gregorian(i, 1, 1),
                alg_p.get_julian_day_from_gregorian(i, 1, 1))

    def test_is_leap_year(self):
        for i in range(3000):
            self.assertEqual(
                alg_c.is_leap_year(i),
                alg_p.is_leap_year(i))

    def test_days_in_year(self):
        for i in range(3000):
            self.assertEqual(
                alg_c.days_in_year(i),
                alg_p.days_in_year(i))

    def test_days_in_month(self):
        for i in range(3000):
            for m in range(1, 13):
                self.assertEqual(
                    alg_c.days_in_month(i, m),
                    alg_p.days_in_month(i, m))

    def test_julian_day_from_jalali_date(self):
        for y in range(303):
            for m in range(1, 13):
                for d in range(1, alg_c.days_in_month(y, m)+1):
                    self.assertEqual(
                        alg_c.julian_day_from_jalali_date(y, m,  d),
                        alg_p.julian_day_from_jalali_date(y, m , d))

    def test_jalali_date_from_julian_day(self):
        jd = 0
        while jd < 365 * 1000:
            jd += 1
            self.assertEqual(
                alg_c.jalali_date_from_julian_day(jd),
                alg_p.jalali_date_from_julian_day(jd))

    def test_gregorian_date_from_julian_day(self):
        jd = 0
        while jd < 365 * 200:
            jd += 1
            self.assertEqual(
                alg_c.gregorian_date_from_julian_day(jd),
                alg_p.gregorian_date_from_julian_day(jd))

    def test_jalali_date_from_gregorian_date(self):
        jd = 0
        while jd < 365 * 200:
            jd += 1
            self.assertEqual(
                alg_c.jalali_date_from_gregorian_date(*alg_c.gregorian_date_from_julian_day(jd)),
                alg_p.jalali_date_from_gregorian_date(*alg_p.gregorian_date_from_julian_day(jd)),)

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
from .persian import PersianNumberDirective
__author__ = 'vahid'


class PersianHour24Directive(PersianNumberDirective):

    def format(self, d):
        return super(PersianHour24Directive, self).format(d.hour)

    def post_parser(self, ctx, formatter):
        super(PersianHour24Directive, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['hour'] = ctx[self.name]


class PersianHour12Directive(PersianNumberDirective):

    def format(self, d):
        return super(PersianHour12Directive, self).format(d.hour12())

    def post_parser(self, ctx, formatter):
        super(PersianHour12Directive, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['hour12'] = ctx[self.name]


class PersianMinuteDirective(PersianNumberDirective):

    def format(self, d):
        return super(PersianMinuteDirective, self).format(d.minute)

    def post_parser(self, ctx, formatter):
        super(PersianMinuteDirective, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['minute'] = ctx[self.name]


class PersianSecondDirective(PersianNumberDirective):

    def format(self, d):
        return super(PersianSecondDirective, self).format(d.second)

    def post_parser(self, ctx, formatter):
        super(PersianSecondDirective, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['second'] = ctx[self.name]


class PersianMicrosecondDirective(PersianNumberDirective):

    def format(self, d):
        return super(PersianMicrosecondDirective, self).format('%.6d' % d.microsecond)

    def post_parser(self, ctx, formatter):
        super(PersianMicrosecondDirective, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['microsecond'] = ctx[self.name]




# -*- coding: utf-8 -*-
import sys
__author__ = 'vahid'
_py3 = sys.version_info.major == 3
if _py3:
    from builtins import range as xrange_compat
else:
    # noinspection PyUnboundLocalVariable
    xrange_compat = xrange

xrange = xrange_compat

def get_unicode(s):
    if _py3:
        return str(s)
    else:
        return unicode(s)

# def xrange(*args, **kw):
#     return xrange_compat(*args, **kw)




# -*- coding: utf-8 -*-
import unittest
from khayyam.timezones import TehranTimezone
from khayyam import JalaliDatetime
from datetime import datetime, timedelta
__author__ = 'vahid'


class TestTehTz(unittest.TestCase):
    
    def setUp(self):
        self.tz = TehranTimezone()
        self.dt = datetime(2000, 1, 1)
        self.dtz = datetime(2000, 1, 1, tzinfo=self.tz)

    def test_dst(self):
        naive_time = JalaliDatetime(1390, 12, 29, 1, 1, 1, 1)
        dst_aware_time = JalaliDatetime(1390, 12, 29, 1, 1, 1, 1, tzinfo=self.tz)
        dst_time = dst_aware_time + timedelta(5)
        self.assertIsNotNone(dst_time.dst(), "invalid dst")
        self.assertEqual(dst_aware_time.dst(), timedelta(0), "invalid dst")
        self.assertIsNone(naive_time.dst(), "invalid dst")

    def test_repr(self):
        t = TehranTimezone()
        self.assertEqual(type(repr(t)), str)
        self.assertEqual(repr(t), '+03:30 dst:60')



if __name__ == '__main__':
    unittest.main()        



# -*- coding: utf-8 -*-
import os
import re
import sys
import warnings
import platform
from setuptools import setup, find_packages, Extension
import traceback
__author__ = 'vahid'


# reading package version (same way sqlalchemy does)
with open(os.path.join(os.path.dirname(__file__), 'khayyam', '__init__.py')) as v_file:
    package_version = re.compile(r".*__version__ = '(.*?)'", re.S).match(v_file.read()).group(1)


if sys.version_info.major == 3:
    def readme(fn):
        with open(fn, encoding='UTF-8') as f:
            return f.read()
else:
    def readme(fn):
        with open(fn) as f:
            return f.read()


WARNING_HEADER = '#' * 40


setup_args = dict(
    name="Khayyam",
    version=package_version,
    author="Vahid Mardani",
    author_email="vahid.mardani@gmail.com",
    url="http://khayyam.dobisel.com",
    description="A cythonic and fast Persian Date & Time library (aka: Jalali Calendar) with timezone, DST"
                "(daylight-saving), full formatting & parsing support for python 2 & 3.",
    zip_safe=True,
    keywords="Khayyam persian jalali date time datetime conversion",
    long_description=readme(os.path.join(os.path.dirname(__file__), 'README.rst')),
    license="GPLv3",
    packages=find_packages(),
    test_suite="khayyam.tests",
    tests_require=[
        'rtl'
    ],
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        "Intended Audience :: Developers",
        "Natural Language :: Persian",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Localization"],
)


def run_setup(with_extensions=True):
    if with_extensions:
        try:
            # noinspection PyPackageRequirements
            from Cython.Build import cythonize
            print('Using Cython to build the extensions.')
            use_cython = True
        except ImportError:
            use_cython = False

            # noinspection PyUnusedLocal
            def cythonize(*a, **kw):
                raise ImportError('Cython is not installed.')

            warnings.warn(
                '\n%s\nNot using Cython to build the extensions.\nUsing available C compiler instead.\n%s' % (
                    WARNING_HEADER, WARNING_HEADER
                ))

        libraries = []

        if platform.system() != 'Windows':
            libraries.append('m')  # Unix-like specific

        extensions = [
            Extension("khayyam.algorithms_c",
                      sources=["khayyam/algorithms_c%s" % ('.pyx' if use_cython else '.c')],
                      libraries=libraries
                      )
        ]

        if use_cython:
            extensions = cythonize(extensions)

        setup_args['ext_modules'] = extensions

    if not with_extensions and 'ext_modules' in setup_args:
        del setup_args['ext_modules']

    setup(**setup_args)


def warning_c_extention():
    warnings.warn(
        '\n%s\n%s\n%s' % (
            WARNING_HEADER,
            "WARNING: The C extension could not be compiled.\n"
            "WARNING: Speedups are not enabled.",
            WARNING_HEADER
        ))


try:
    run_setup()

# noinspection PyBroadException
except:
    traceback.print_exc()
    warning_c_extention()
    run_setup(False)
    warning_c_extention()



# -*- coding: utf-8 -*-
import warnings

__author__ = 'vahid'

try:
    from .algorithms_c import \
        is_leap_year, \
        days_in_year, \
        days_in_month, \
        get_julian_day_from_gregorian, \
        julian_day_from_jalali_date, \
        jalali_date_from_julian_day, \
        gregorian_date_from_julian_day, \
        jalali_date_from_gregorian_date

except ImportError:
    warnings.warn(
        "The Cython extension is not available. Switching to fallback python pure algorithms, "
        "so it's may be slower than C implementation of the algorithms.")
    from .algorithms_pure import \
        is_leap_year, \
        days_in_year, \
        days_in_month, \
        get_julian_day_from_gregorian, \
        julian_day_from_jalali_date, \
        jalali_date_from_julian_day, \
        gregorian_date_from_julian_day, \
        jalali_date_from_gregorian_date






# -*- coding: utf-8 -*-
from khayyam.compat import get_unicode
__author__ = 'vahid'


class Directive(object):
    def __init__(self, key, name, regex, type_, formatter=None, post_parser=None):
        self.key = key
        self.name = name
        self.regex = regex
        self.type_ = type_
        if formatter:
            self.format = formatter
        if post_parser:
            self.post_parser = post_parser

    def __repr__(self):
        return '%' + self.key

    def post_parser(self, ctx, formatter):
        pass

    def format(self, d):
        return d


class CompositeDateDirective(Directive):
    format_string = None

    def __init__(self, key, name, regex, format_string=None, **kw):
        if format_string:
            self.format_string = format_string
            self._sub_formatter = None
        super(CompositeDateDirective, self).__init__(key, name, regex, get_unicode, **kw)

    def _create_formatter(self):
        from khayyam.formatting import JalaliDateFormatter
        return JalaliDateFormatter(self.format_string)

    @property
    def sub_formatter(self):
        if not self._sub_formatter:
            self._sub_formatter = self._create_formatter()
        return self._sub_formatter

    def format(self, d):
        return d.strftime(self.format_string)

    def post_parser(self, ctx, formatter):
        ctx.update(self.sub_formatter.parse(ctx[self.name]))


class CompositeDatetimeDirective(CompositeDateDirective):
    format_string = None

    def _create_formatter(self):
        from khayyam.formatting import JalaliDatetimeFormatter
        return JalaliDatetimeFormatter(self.format_string)


# -*- coding: utf-8 -*-
import re
from .directive import Directive
from .persian import PersianNumberDirective, persian_to_eng
from khayyam.formatting import constants as consts
from khayyam.compat import get_unicode
from datetime import timedelta
from khayyam.timezones import Timezone
__author__ = 'vahid'


class UTCOffsetDirective(Directive):

    def format(self, d):
        if not d.tzinfo:
            return ''
        seconds = d.utcoffset().total_seconds()
        return '%s%.2d:%.2d' % (
            '+' if seconds >= 0 else '-',
            int(seconds / 3600),
            int((seconds % 3600) / 60),
        )

    def post_parser(self, ctx, formatter):
        exp = ctx[self.name]
        if exp.strip() == '':
            return
        regex = '(?P<posneg>[-+]?)(?P<hour>\d{2}):(?P<minute>\d{2})'
        match = re.match(regex, exp)
        d = match.groupdict()
        posneg = lambda i: 0 - i if d['posneg'] == '-' else i
        hours = int(d['hour'])
        minutes = int(d['minute'])
        if not hours and not minutes:
            return
        ctx.update(dict(tzinfo=Timezone(timedelta(
            hours = posneg(hours),
            minutes = posneg(minutes)
        ))))



class PersianUTCOffsetDirective(PersianNumberDirective):

    def format(self, d):
        if not d.tzinfo:
            return ''
        seconds = d.utcoffset().total_seconds()
        return super(PersianUTCOffsetDirective, self).format('%s%.2d:%.2d' % (
            '+' if seconds >= 0 else '-',
            int(seconds / 3600),
            int((seconds % 3600) / 60),
        ))

    def post_parser(self, ctx, formatter):
        exp = ctx[self.name]
        if exp.strip() != '':
            ctx['utcoffset'] = persian_to_eng(exp)


class TimezoneNameDirective(Directive):

    def format(self, d):
        if d.tzinfo:
            return d.tzname()
        return ''



# -*- coding: utf-8 -*-
from .directive import Directive
from .persian import PersianNumberDirective
from khayyam.algorithms_pure import days_in_year
from datetime import timedelta
__author__ = 'vahid'


class PersianDayDirective(PersianNumberDirective):

    def format(self, d):
        return super(PersianDayDirective, self).format(d.day)

    def post_parser(self, ctx, formatter):
        super(PersianDayDirective, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['day'] = ctx[self.name]


class DayOfYearDirective(Directive):

    def format(self, d):
        return '%.3d' % d.dayofyear()

    def post_parser(self, ctx, formatter):
        _dayofyear = ctx['dayofyear']
        if 'year' not in ctx:
            ctx['year'] = 1
        if 'month' in ctx:
            del ctx['month']
        if 'day' in ctx:
            del ctx['day']

        max_days = days_in_year(ctx['year'])
        if _dayofyear > max_days:
            raise ValueError(
                'Invalid dayofyear: %.3d for year %.4d. Valid values are: 1-%s' % (
                    _dayofyear,
                    ctx['year'],
                    max_days))
        from khayyam import JalaliDate
        d = JalaliDate(year=ctx['year']) + timedelta(days=_dayofyear-1)
        ctx.update(dict(
            month=d.month,
            day=d.day
        ))


class PersianDayOfYearDirective(PersianNumberDirective):

    def format(self, d):
        return super(PersianDayOfYearDirective, self).format(d.dayofyear())

    def post_parser(self, ctx, formatter):
        super(PersianDayOfYearDirective, self).post_parser(ctx, formatter)
        if self.name in ctx and ctx[self.name]:
            ctx['dayofyear'] = ctx[self.name]
